@@ -35,9 +35,18 @@
  * that it could be modified to be more general without too much effort.
  * In particular, many of the routines called by ftpProcessReply could
  * be made into external interface functions. Volunteers?
+ *
+ * 28 Apr 1993: Status mesage for bytes transferred was backwards.
+ *  1 Jun 1993: Fixes for ISC. Are these needed for other SYS5?
+ * 30 Jun 1993: Fixes for Encore Umax (used cpp symbol "umax" as a guess).
+ * 24 Aug 1993: Allow 200 reply for CWD as well as 250.
+ *              Non-blocking IO defs for NeXT.
  */
 #include <stdio.h>
 #include <errno.h>
+#ifndef MSDOS
+extern int errno;	/* Encore needs this despite <errno.h> */
+#endif
 #include "config.h"
 #ifdef HAVE_SYS_PARAM_H
 #include <sys/param.h>
@@ -59,8 +68,10 @@ extern void sysError(/* str */);
  * Portable non-blocking I/O macros, I hope.
  *   hp300 : From Andy.Linton@comp.vuw.ac.nz
  *   sgi   : From amoss@cs.huji.ac.il
+ *   umax  : From Gerry.Tomlinson@newcastle.ac.uk
+ *   NeXT  : From jk@zarniwoop.pc-labor.uni-bremen.de
  */
-#if defined(hp300)
+#if defined(hp300) || defined(NeXT)
   /* Here's for BSD, maybe, but Sys5 can't tell this from EOF. */
 # include <fcntl.h>
 # define MAKE_NONBLOCKING(fd) fcntl(fd,F_SETFL,O_NDELAY)
@@ -70,13 +81,18 @@ extern void sysError(/* str */);
 # include <fcntl.h>
 # define MAKE_NONBLOCKING(fd) fcntl(fd,F_SETFL,O_NONBLOCK)
 # define ITWOULDBLOCK EWOULDBLOCK
+#else
+#if defined(umax)
+# define MAKE_NONBLOCKING(fd) fcntl(fd,F_SETFL,FNDELAY)
+# define ITWOULDBLOCK EWOULDBLOCK
 #else
   /* This is POSIX, the default, which uses EAGAIN. */
 # include <fcntl.h>
 # define MAKE_NONBLOCKING(fd) fcntl(fd,F_SETFL,O_NONBLOCK)
 # define ITWOULDBLOCK EAGAIN
-#endif
-#endif
+#endif /* umax */
+#endif /* sgi */
+#endif /* hp300 */
 /*
  * Networking includes from Brendan Kehoe (dirsend.c)
  */
@@ -440,8 +456,16 @@ FtpContext *ftpc;
 	return(-1);
     }
     DEBUG1("ftpGetSocket: socket() returned %d\n",ftpc->ctrl);
+    /*
+     * According to jbh@moses.aii.COM, we can't do this until we connect
+     * on (at least) ISC variants of SYS5. I have not had problems with
+     * other SYS5 systems, including HPUX. When ISC is defined, we do it
+     * in ftpConnect(), although we may then block in connect().
+     */
+#ifndef ISC
     /* Setup socket for async i/o */
     MAKE_NONBLOCKING(ftpc->ctrl);
+#endif
     /* Set up the address spec. */
     bzero((char *)&(ftpc->saddr),sizeof(struct sockaddr_in));
     ftpc->saddr.sin_family = AF_INET;
@@ -483,6 +507,10 @@ FtpContext *ftpc;
 	  fprintf(stderr,"ftpConnect: connect() EISCONN\n");
       else
 	  fprintf(stderr,"ftpConnect: connect() returned ok\n");
+#endif
+     /* According to jbh@moses.aii.COM, see comments in ftpGetSocket(). */
+#ifdef ISC
+    MAKE_NONBLOCKING(ftpc->ctrl); /* Setup socket for async i/o */
 #endif
     /* Get name (address) of socket */
     addrlen = sizeof(struct sockaddr_in);
@@ -746,7 +774,8 @@ FtpContext *ftpc;
 	break;
       case FTPS_CWD:
 	/* can come here direct from USER or PASS also... */
-	if (ftpc->retcode == FTP_FILE_ACTION_OK) {	/* last CWD ok */
+	if (ftpc->retcode == FTP_FILE_ACTION_OK ||
+	    ftpc->retcode == FTP_COMMAND_OK) {	/* last CWD ok */
 	    if (ftpc->wd == NULL || *(ftpc->wd) == '\0') { /* CWD done */
 		ftpc->state = FTPS_TYPE;
 		ftpSendType(ftpc);
@@ -1213,7 +1242,7 @@ FtpContext *ftpc;
 	    DEBUG1("ftpReadData: total = %d\n",ftpc->num_bytes);
 	    if (ftpc->this_size != 0) {
 		sprintf(msg,"%.200s: %d/%d",ftpc->files[ftpc->this_file],
-			(char *)(ftpc->this_size),(char *)(ftpc->num_bytes));
+			(char *)(ftpc->num_bytes),(char *)(ftpc->this_size));
 	    } else {
 		sprintf(msg,"%.200s: %d bytes",ftpc->files[ftpc->this_file],
 			(char *)(ftpc->num_bytes));
