@@ -2,6 +2,7 @@
  * tilde.c : Tilde expansion for filenames
  *
  * George Ferguson, ferguson@cs.rochester.edu, 23 Apr 1993.
+ * 13 May 1993: Cleanups.
  *
  */
 
@@ -13,8 +14,9 @@
 # include <sys/param.h>
 #endif
 #include "alert.h"
+#include "sysdefs.h"
+#include "stringdefs.h"
 extern uid_t getuid();		/* not in stdlib.h, but is in unistd.h */
-extern char *getenv();
 
 char *
 tildeExpand(file)
@@ -40,7 +42,7 @@ char *file;
 	if ((pwe=getpwuid(getuid())) != NULL) {
 	    home = pwe->pw_dir;
 	} else if ((home=getenv("HOME")) == NULL) {
-	    alert1("Couldn't find homedir, you should set $HOME");
+	    alert0("Couldn't find homedir, you should set $HOME");
 	    return(filename);
 	}
     } else {						/* ~user/... */
